package ch03.jdbc;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;
import java.util.SortedSet;
import java.util.TreeSet;

/**
 * Beispielklasse zum einfacheren Zugriff auf Datenbankverbindungseinstellungen
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public final class DbProperties
{
    private static final String   URL           = "db.url";
    private static final String   USER_NAME     = "db.username";
    private static final String   PASSWORD      = "db.password";

    private static final String[] REQUIRED_KEYS = { URL, USER_NAME, PASSWORD };

    private static final String   FILENAME      = "config/db.properties";

    private final String          filePath;
    private final Properties      properties    = new Properties();

    public DbProperties()
    {
        final File file = new File(FILENAME);
        filePath = file.getAbsolutePath();
        
        try (final InputStream inputStream = new BufferedInputStream(
        		                             new FileInputStream(file)))
        {
            properties.load(inputStream);

            ensureAllKeysAvailable();
            
            // check if the required url value is given 
            final String url = properties.getProperty(URL, "");
            if (url.isEmpty())
            {
                throw new IllegalStateException("db config file '" + filePath + "' is incomplete! "
                                                + "Missing value for key: '" + URL + "'");
            }
        }
        catch (final IOException ioException)
        {
            throw new IllegalStateException("problems while accessing " + 
                                            "db config file '" + filePath + "'", 
                                            ioException);

            // Ressourcen werden automatisch durch ARM wieder freigegeben  
        }
    }

    private void ensureAllKeysAvailable()
    {
        final SortedSet<String> missingKeys = new TreeSet<>();

        for (final String key : REQUIRED_KEYS)
        {
            if (!properties.containsKey(key))
            {
                missingKeys.add(key);
            }
        }

        if (!missingKeys.isEmpty())
        {
            throw new IllegalStateException("db config file '" + filePath + "' " + 
                                            "is incomplete! Missing keys: " + missingKeys);
        }
    }

    public String getUrl()
    {
    	return properties.getProperty(URL, "");
    }

    public String getUserName()
    {
        return properties.getProperty(USER_NAME, "");
    }

    public String getPassword()
    {
        return properties.getProperty(PASSWORD, "");
    }
}
