package ch03.jdbc;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.TableModel;

/**
 * Beispielklasse, die dynamische Abfragen erlaubt und dazu ein Tabellenmodell auf Basis eines ResultSets nutzt.
 * 
 * @author Michael Inden
 * 
 * Copyright 2012, 2016 by Michael Inden 
 */
public final class MetaDataQueryFrameExample
{
    public static void main(final String[] args) throws SQLException
    {
        Connection connection = null;
        // Hier kein ARM, da keine Freigabe am Blockende erlaubt, s.u.
        try
        {
            final DbProperties dbProperties = new DbProperties();
            connection = DriverManager.getConnection(dbProperties.getUrl(), dbProperties.getUserName(), dbProperties.getPassword());
            final Statement stmt = connection.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
            final MetaDataQueryFrame frame = new MetaDataQueryFrame(stmt);
            frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
            frame.setSize(500, 300);
            frame.setVisible(true);
            
            // Trick: Zugriff auf die nicht finale Connection
            // in der inneren Klasse ermöglichen
            final Connection connectionReference = connection;
            frame.addWindowListener(new WindowAdapter()
            {
                @Override
                public void windowClosing(final WindowEvent event)
                {
                    DbUtils.safeCloseStatement(stmt);
                    DbUtils.safeCloseConnection(connectionReference);
                    frame.setVisible(false);
                    frame.dispose();
                }
            });
            // initial "Execute SQL" ausführen
            frame.executeSqlSelect();
        }
        catch (final SQLException e)
        {
            DbUtils.safeCloseConnection(connection);
            throw e;
        }
        finally
        {
            // Freigabe der Ressourcen hier nicht möglich, da
            // zu früh, GUI läuft ja noch
            // DbUtils.safeCloseConnection(connection);
        }
    }

    public static class MetaDataQueryFrame extends JFrame
    {
        private static final String INITIAL_SELECT = "SELECT Vorname, " + "Name AS Nachname," + " Geburtstag FROM Personen";
        // Könnten über RessourceManager ermittelt werden
        private static final String EXECUTE_SQL = "Execute SQL";
        private static final String ENTER_SQL_SELECT = "Enter SQL SELECT:";
        // Diese Bedienelemente müssen bekannt sein.
        private final JButton executeSqlButton = new JButton(EXECUTE_SQL);
        private final JTable table = new JTable();
        private final JTextField sqlInfoTextField = new JTextField();

        public MetaDataQueryFrame(final Statement stmt) 
        {
            super("ResultSetMetaDataExample");
            buildGUI(stmt);
        }

        private void buildGUI(final Statement stmt)
        {
            add(createSqlEnterContent(stmt), BorderLayout.NORTH);
            add(createResultTableContent(), BorderLayout.CENTER);
            add(createInfoContent(), BorderLayout.SOUTH);
        }

        private void executeSqlSelect()
        {
            executeSqlButton.doClick();
        }

        private JComponent createSqlEnterContent(final Statement stmt)
        {
            final JLabel enterSqlLabel = new JLabel(ENTER_SQL_SELECT);
            final JTextField sqlCommandTextField = new JTextField(30);
            sqlCommandTextField.setText(INITIAL_SELECT);
            final JPanel sqlExecutePanel = new JPanel(new BorderLayout());
            sqlExecutePanel.add(enterSqlLabel, BorderLayout.WEST);
            sqlExecutePanel.add(sqlCommandTextField, BorderLayout.CENTER);
            sqlExecutePanel.add(executeSqlButton, BorderLayout.EAST);
            executeSqlButton.addActionListener(new ExecuteSqlActionListener(sqlCommandTextField, sqlInfoTextField, table, stmt));
            return sqlExecutePanel;
        }

        private JComponent createResultTableContent()
        {
            return new JScrollPane(table);
        }

        private JComponent createInfoContent()
        {
            sqlInfoTextField.setEditable(false);
            return sqlInfoTextField;
        }
    }

    public static class ExecuteSqlActionListener implements ActionListener
    {
        private static final String ROW_COUNT_TEXT = "Row count: ";
        private final JTextField sqlCommandTextField;
        private final JTextField sqlInfoTextField;
        private final JTable table;
        private final Statement statement;
        ResultSet currentResultSet = null;

        public ExecuteSqlActionListener(final JTextField sqlCommandTextField, final JTextField sqlInfoTextField, final JTable table,
                        final Statement statement) {
            this.sqlCommandTextField = sqlCommandTextField;
            this.sqlInfoTextField = sqlInfoTextField;
            this.table = table;
            this.statement = statement;
        }

        @Override
        public void actionPerformed(final ActionEvent event)
        {
            final String sqlSelect = sqlCommandTextField.getText();
            try
            {
                // ResultSet freigeben, falls bereits eins belegt war
                DbUtils.safeCloseResultSet(currentResultSet);
                currentResultSet = statement.executeQuery(sqlSelect);
                final TableModel tableModel = new ResultSetTableModel(currentResultSet);
                table.setModel(tableModel);
                sqlInfoTextField.setText(ROW_COUNT_TEXT + tableModel.getRowCount());
            }
            catch (final SQLException e1)
            {
                sqlInfoTextField.setText(e1.getLocalizedMessage());
            }
        }
    }
}