package ch04.jpa.jpql_queries;

import java.util.Date;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Path;
import javax.persistence.criteria.Root;

/**
 * Beispiel zur Demonstration von Abfragen mit dem Criteria API
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public final class PersonJpqlAndCriteriaApiExample
{
    public static void main(final String[] args) throws Exception
    {
        final EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory("java-profi-PU-JPQL");
        final EntityManager entityManager = entityManagerFactory.createEntityManager();

        try
        {
            performJPAActions(entityManager);
        }
        finally
        {
            entityManager.close();
            entityManagerFactory.close();
        }
    }

    private static void performJPAActions(final EntityManager entityManager)
    {
        try
        {
            entityManager.getTransaction().begin();

            executeStatements(entityManager);

            entityManager.getTransaction().commit();
        }
        catch (Exception e)
        {
            entityManager.getTransaction().rollback();
        }
    }

    private static void executeStatements(final EntityManager entityManager)
    {
        createPersons(entityManager);

        calcAvgAge(entityManager);
        personsWithinAgeRange(entityManager, 40, 50);
    }

    private static void createPersons(final EntityManager entityManager)
    {
        final Person micha = new Person("Micha-JPQL", "Inden", new Date(71, 1, 7));
        final Person tim = new Person("Tim-JPQL", "Bötz", new Date(84, 2, 27));
        final Person tom = new Person("Tom-JPQL", "Meyer", new Date(55, 2, 31));

        entityManager.persist(micha);
        entityManager.persist(tim);
        entityManager.persist(tom);
    }

    private static void calcAvgAge(final EntityManager entityManager)
    {
        final String avgAgeJPQL = "SELECT AVG(person.age) FROM Person person";
        final TypedQuery<Double> query1 = entityManager.createQuery(avgAgeJPQL, Double.class);

        final CriteriaBuilder criteriaBuilder = entityManager.getCriteriaBuilder();
        final CriteriaQuery<Double> criteriaQuery = criteriaBuilder.createQuery(Double.class);
        final Root<Person> person = criteriaQuery.from(Person.class);
        criteriaQuery.select(criteriaBuilder.avg(person.get("age")));
        final TypedQuery<Double> query2 = entityManager.createQuery(criteriaQuery);

        final Double avgAge1 = query1.getSingleResult();
        final Double avgAge2 = query2.getSingleResult();
        System.out.println("\nAvg Age 1: " + avgAge1 + " / Age 2: " + avgAge2);
    }

    private static void personsWithinAgeRange(final EntityManager entityManager, final int minAge, final int maxAge)
    {
        final String selectJPQL = "SELECT person FROM Person person "
                                  + "WHERE person.age >= :minAge AND person.age < :maxAge";
        final TypedQuery<Person> typedQuery1 = entityManager.createQuery(selectJPQL, Person.class);
        typedQuery1.setParameter("minAge", minAge);
        typedQuery1.setParameter("maxAge", maxAge);

        final CriteriaBuilder criteriaBuilder = entityManager.getCriteriaBuilder();
        final CriteriaQuery<Person> criteriaQuery = criteriaBuilder.createQuery(Person.class);
        final Root<Person> person = criteriaQuery.from(Person.class);
        final Path<Integer> age = person.get("age");
        criteriaQuery.where(criteriaBuilder.and(criteriaBuilder.greaterThanOrEqualTo(age, minAge),
                                                criteriaBuilder.lessThan(age, maxAge)));
        final TypedQuery<Person> typedQuery2 = entityManager.createQuery(criteriaQuery);

        final List<Person> results1 = typedQuery1.getResultList();
        final List<Person> results2 = typedQuery2.getResultList();
        System.out.println("Results1: " + results1);
        System.out.println("Results2: " + results2);
    }
}
