package ch05.mongo;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.function.Consumer;
import java.util.stream.Collectors;

import org.bson.Document;

import com.mongodb.MongoClient;
import com.mongodb.client.FindIterable;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoDatabase;

/**
 * Beispielklasse zum Verwalten kompleerer Daten in Form von Adressen und Personen
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden
 */
public class MongoPersonsAndAddressExample
{
    public static void main(final String[] args)
    {
        try (final MongoClient mongo = new MongoClient("localhost", 27017))
        {
            final MongoDatabase db = mongo.getDatabase("mongoexample");
            final MongoCollection<Document> collection = db.getCollection("personsAndAddresses");

            final List<Document> persons = createPersonDocuments();
            insertDocuments(collection, persons);
            countAllPersons(collection);
            queryAll(collection);

            final int countBefore = countPersonsOlderThan(collection, 40);
            System.out.println("Person with age > 40 --> " + countBefore);

            updatePersonsAgeOlderThanTo(collection, 40, 25);

            final int countAfter = countPersonsOlderThan(collection, 40);
            System.out.println("Person with age > 40 --> " + countAfter);
        }
    }

    private static List<Document> createPersonDocuments()
    {
        return ExampleData.names.stream().map(name -> createPersonDocument(name)).collect(Collectors.toList());
    }

    private static void insertDocuments(final MongoCollection<Document> collection,
                                        final List<Document> dbObjectsToInsert)
    {
        dbObjectsToInsert.forEach((document) -> collection.insertOne(document));
    }

    private static void countAllPersons(final MongoCollection<Document> collection)
    {
        System.out.println("All Persons: " + collection.count());
    }

    private static void queryAll(final MongoCollection<Document> collection)
    {
        final FindIterable<Document> results = collection.find();
        printAll(results);
    }

    private static int countPersonsOlderThan(final MongoCollection<Document> collection, final int minAge)
    {
        final Document query = new Document("age", new Document("$gt", minAge));

        final FindIterable<Document> result = collection.find(query);

        return result.into(new ArrayList<>()).size();
    }

    private static void updatePersonsAgeOlderThanTo(final MongoCollection<Document> collection, final int minAge,
                                                    final int newAge)
    {
        final Document query = new Document("age", new Document("$gt", minAge));

        final Document newDocument = new Document("age", newAge);
        final Document update = new Document("$set", newDocument);

        collection.updateMany(query, update);
    }

    private static void printAll(final FindIterable<Document> elements)
    {
        final Consumer<Document> consumer = System.out::println;
        elements.forEach(consumer);
    }

    private static Document createPersonDocument(final String name)
    {
        final Document document = new Document();
        document.append("name", name);
        document.append("age", 1 + (int) (Math.random() * 100));
        document.append("insertionDate", new Date());
        // Support für Array und eingebettete Dokumente
        document.append("hobbies", Arrays.asList(createHobbiesArray()));
        document.append("address", createAddressDocument());
        return document;
    }

    private static String[] createHobbiesArray()
    {
        final int maxNrOfHobbies = 1 + (int) (Math.random() * (ExampleData.hobbies.size() / 2));

        return ExampleData.hobbies.stream().filter(item -> Math.random() > 0.5).limit(maxNrOfHobbies)
                        .toArray(String[]::new);
    }

    private static Document createAddressDocument()
    {
        final int random = (int) (Math.random() * ExampleData.addresses.size());
        final ExampleData.Address address = ExampleData.addresses.get(random);

        final Document document = new Document();
        document.append("country", address.state);
        document.append("zipCode", address.zipCode);
        document.append("city", address.city);
        return document;
    }
}