package ch06.rest.jersey.server.basics;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.IntStream;

import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.DefaultValue;
import javax.ws.rs.FormParam;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.GenericEntity;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

/**
 * Beispielklasse zur Definition einer REST-Ressource, die Nachrichten vom Typ Message verwaltet
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden
 */
@Path("/messages")
public class MessageResource
{
    private static final List<Message> messages = new ArrayList<>();
    static
    {
        IntStream.range(0, 8).forEach(i -> messages.add(new Message("Test" + i)));
    }

    @GET
    @Path("/{id}")
    @Produces({ MediaType.APPLICATION_XML, MediaType.APPLICATION_JSON })
    public Response getMessageById(@PathParam("id") final long id)
    {
        final Optional<Message> optionalMsg = findMessageBy(id);

        if (optionalMsg.isPresent())
        {
            final Message msg = optionalMsg.get();
            return Response.status(Response.Status.OK).entity(msg).build();
        }

        return Response.status(Response.Status.NOT_FOUND).build();
    }

    private Optional<Message> findMessageBy(final long id)
    {
        return messages.stream().filter(msg -> msg.getId() == id).findFirst();
    }

    @POST
    @Consumes(MediaType.APPLICATION_FORM_URLENCODED)
    public Response createMessage(@FormParam("msg") final String msgText)
    {
        if (msgText.trim().length() > 0)
        {
            // Vereinfachend wird nur eine neue Nachricht erzeugt            
            // In der Realität würde diese vermutlich in einer Datenbank persistiert
            final Message msg = new Message(msgText);
            messages.add(msg);

            return Response.created(URI.create("/rest/messages/" + msg.getId())).build();
        }
        return Response.status(Response.Status.BAD_REQUEST).build();
    }

    @GET
    @Consumes({ MediaType.APPLICATION_XML, MediaType.APPLICATION_JSON })
    public Response getMessages(@QueryParam("offset") @DefaultValue("0") int offset,
                                @QueryParam("limit") @DefaultValue("20") int limit)
    {
        final int max = Math.min(offset+limit, messages.size());
        final List<Message> subList = messages.subList(offset, max);
   
        final GenericEntity<List<Message>> entityList = 
                        new GenericEntity<List<Message>>(subList) {};
                        
        return Response.status(Response.Status.OK).entity(entityList).build();
    }
    
    @PUT
    @Path("/{id}")
    @Consumes({ MediaType.APPLICATION_XML, MediaType.APPLICATION_JSON })
    public Response updateMessage(@PathParam("id") final long id, final Message msg)
    {
        final Optional<Message> optionalMsg = findMessageBy(id);

        if (optionalMsg.isPresent())
        {
            final Message origMsg = optionalMsg.get();

            // Aktualisierung der gewünschten Attribute
            origMsg.setMsg(msg.getMsg());
            origMsg.setCreationDate(msg.getCreationDate());
        }
        else
        {
            messages.add(msg);
        }
        return Response.created(URI.create("/rest/messages/" + msg.getId())).build();
    }

    @DELETE
    @Path("/{id}")
    public Response deleteMessageById(@PathParam("id") final long id)
    {
        final Optional<Message> optionalMsg = findMessageBy(id);

        if (optionalMsg.isPresent())
        {
            final Message msg = optionalMsg.get();
            messages.remove(msg);

            return Response.noContent().build();
        }

        return Response.status(Response.Status.NOT_FOUND).build();
    }
}