package ch07.exampleapp.highscores.iteration0;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.logging.Logger;

/**
 * Beispiel zur Verarbeitung von CVS-Dateien
 * <br>
 * Lese aus einer speziellen csv-Datei alle Werte ein und mache daraus eine Liste 
 * von Highscore-Objekten. Dabei sollte jede Zeile der Datei den folgenden Aufbau 
 * haben: "<code>Name , Punkte , Level , Datum</code>"
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public final class HighscoresCsvImporter
{
    private static final Logger log = Logger.getLogger("HighscoresCsvImporter");

    private static final int VALUE_COUNT = 4;

    public static List<Highscore> readHighscoresFromCsv(final String fileName)
    {
        final List<Highscore> highscores = new ArrayList<>();

        try
        {
        	// JDK 8: readAllLines(), forEach() & Lambda
	        final List<String> lines = Files.readAllLines(Paths.get(fileName), StandardCharsets.ISO_8859_1);
	        lines.forEach(line ->
            {
            	// JDK 8: Optional & Lambda
                final Optional<Highscore> optionalHighscore = extractHighscoreFromLine(line);
                optionalHighscore.ifPresent(highscore -> highscores.add(highscore));
            });
	    }
        catch (final IOException e)
        {
            log.warning("processing of file '" + fileName + "' failed: " + e);
        }

        return highscores;
    }

private static Optional<Highscore> extractHighscoreFromLine(final String line)
{
    // Spalte die Eingabe mit ';' oder ',' auf           
    final String[] values = line.split(";|,");

    // Behandlung von Leerzeilen und Kommentaren            
    if (isEmptyLineOrComment(values) || values.length != VALUE_COUNT)
    {
        return Optional.empty();
    }

    try
    {
        // Auslesen der Werte als String + Typprüfung + Konvertierung 
        final String name = values[0].trim();
        final int points = Integer.parseInt(values[1].trim());
        final int level = Integer.parseInt(values[2].trim());
        final String dateAsString = values[3].trim();
        
        final DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("dd.MM.yyyy");
        final LocalDate day = LocalDate.parse(dateAsString, dateTimeFormatter);

        return Optional.of(new Highscore(name, points, level, day));
    }
    catch (final NumberFormatException e)
    {
        log.warning("Skipping invalid point or level value '" + line + "'");
    }
    catch (final DateTimeParseException e)
    {
        log.warning("Skipping invalid date value '" + line + "'");
    }
    return Optional.empty();
}

private static boolean isEmptyLineOrComment(final String[] values)
{
    return (values.length == 1 && (values[0].trim().length() == 0) || 
		   // Ignoriere Kommentare, die auch ',' oder ';' enthalten
		   (values.length >= 1 && values[0].trim().startsWith("#")));
}    
}
