package ch07.exampleapp.highscores.iteration3;

import static ch07.exampleapp.highscores.iteration3.HighscoresFromCsvImporter.readHighscoresFromCsv;
import static java.util.Comparator.comparing;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Optional;
import java.util.function.Predicate;

/**
 * Beispielklasse zur Demonstration der Verwaltung von Highscores-Daten
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public class HighscoreManager
{
    final Comparator<Highscore>   byPoints            = comparing(Highscore::getPoints);

    final Comparator<Highscore>   byDay               = comparing(Highscore::getDay);

    final Comparator<Highscore>   byLevel             = comparing(Highscore::getLevel);

    final Comparator<Highscore>   byPointsLevelAndDay = byPoints.reversed().thenComparing(byLevel.reversed())
                    .thenComparing(byDay);

    private final List<Highscore> highscores          = new ArrayList<>();

    public List<Highscore> getHighscores()
    {
        // Kopie erzeugen: keine internen Daten herausgeben
        final List<Highscore> soredResults = new ArrayList<>(highscores);
        soredResults.sort(byPointsLevelAndDay);
        return soredResults;
    }

    public void add(final Highscore highscore)
    {
        highscores.add(highscore);
        highscores.sort(byPointsLevelAndDay);

        final List<Highscore> first50 = truncateToMax(highscores, 50);
        populateFrom(first50);
    }

    private List<Highscore> truncateToMax(final List<Highscore> list, final int n)
    {
        final int max = Math.min(n, list.size());

        return new ArrayList<>(list.subList(0, max));
    }

    private void populateFrom(final List<Highscore> first50)
    {
        highscores.clear();
        highscores.addAll(first50);
    }

    public List<Highscore> getHighscoresTopN(final int n)
    {
        final List<Highscore> allHighscores = getHighscores();
        return truncateToMax(allHighscores, n);
    }

    public void delete(final Highscore highscore)
    {
        highscores.remove(highscore);
    }

    public Optional<Highscore> findBy(final long id)
    {
        final Predicate<Highscore> sameId = highscore -> highscore.getId() == id;
        return highscores.stream().filter(sameId).findFirst();
    }

    public void populateFromCsv(final String filePath)
    {
        final List<Highscore> readHighscores = readHighscoresFromCsv(filePath);

        populateFrom(readHighscores);
    }

    public void saveTo(final PersistMode persistMode, final OutputStream os) throws IOException
    {
        final HighscorePersister persister = new HighscorePersister();
        persister.persist(persistMode, os, highscores);
    }

    public void loadFrom(final PersistMode persistMode, final InputStream is) throws IOException
    {
        final HighscorePersister persister = new HighscorePersister();
        final List<Highscore> readHighscores = persister.readFrom(persistMode, is);

        populateFrom(readHighscores);
    }
}